﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml;

namespace FileTree
{
   public partial class TreeForm : Form
   {
      private MySorter sorter = new MySorter();

      private Bitmap picture = null;

      private void listView_ColumnClick(object sender, ColumnClickEventArgs e)
      {
          if (sorter.column_index == e.Column)
          {
              sorter.inv = ! sorter.inv;
          }
          else
          {
              sorter.column_index = e.Column;
              sorter.inv = false;
          }
          listView.Sort(); // zobrazit v novem poradi
      }

      public TreeForm()
      {
         InitializeComponent ();

         listView.ListViewItemSorter = sorter;
         listView.Sorting = SortOrder.Ascending; // nezbytne

         runButton_Click (null, null);

         // dataBox.ColumnCount = 3;
         // string [] pole = { "a", "b", "cc" };
         // dataBox.Rows.Add(pole);
      }

      private void runButton_Click (object sender, EventArgs e)
      {
         DriveInfo [] drives = DriveInfo.GetDrives ();
         foreach ( DriveInfo d in drives )
         {
            MyTreeNode n = new MyTreeNode ();
            n.Text = d.Name;
            n.Path = d.Name;
            try
            {
               // n.ToolTipText = d.VolumeLabel;
            }
            catch ( Exception ex )
            {
               n.ToolTipText = ex.Message;
            }
            treeView.Nodes.Add (n);

            // addSubdirs (n, 2);
         }
      }

      private void addSubdirs (MyTreeNode top, int level)
      {
         if (top.ready)
         {
            if (level > 1)
               foreach (MyTreeNode n in top.Nodes)
                  addSubdirs (n, level-1);
         }
         else
         {
            top.ready = true; // nezapomenout
            try
            {
               DirectoryInfo top_dir = new DirectoryInfo (top.Path);
               DirectoryInfo [] subdirs = top_dir.GetDirectories ();
               foreach (DirectoryInfo d in subdirs)
               {
                  MyTreeNode n = new MyTreeNode ();
                  n.Text = d.Name;
                  n.ToolTipText = d.FullName;
                  n.Path = d.FullName;
                  top.Nodes.Add (n);
                  // showStatus (n.Path);

                  if ( level > 1 )
                     addSubdirs (n, level-1);
               }
            }
            catch (Exception ex)
            {
               // top.ToolTipText = ex.Message;
               // showStatus (ex.Message);
            }
         }
      }

      private void showFiles (MyTreeNode top)
      {
         listView.Items.Clear();
         try
         {
            DirectoryInfo top_dir = new DirectoryInfo (top.Path);
            FileInfo [] files = top_dir.GetFiles ();
            foreach (FileInfo f in files)
            {
               MyListNode n = new MyListNode ();
               n.FilePath = f.FullName;
               n.FileName = f.Name;
               n.FileSize = f.Length;
               n.FileTime = f.LastWriteTime;

               n.Text = n.FileName;
               n.ToolTipText = n.FilePath;

               n.SubItems.Add("" + n.FileSize);

               DateTime t = n.FileTime;
               // n.SubItems.Add(t.ToString("u"));
               n.SubItems.Add(t.ToString("dd-MM-yyyy HH:mm:ss"));

               listView.Items.Add (n);
            }
         }
         catch ( Exception ex )
         {
            // top.ToolTipText = ex.Message;
            // showStatus (ex.Message);
         }
      }

      private void showFile (MyListNode n)
      {
          if (n != null)
          {
              string ext = Path.GetExtension (n.FilePath);
              ext = ext.ToLower ();
              switch (ext)
              {
                  case ".txt":
                  case ".bat":
                  case ".log":
                  case ".pas":
                  case ".c":
                  case ".cpp":
                      textBox.LoadFile (n.FilePath, RichTextBoxStreamType.PlainText);
                      tabControl.SelectedTab = textTab;
                      break;

                  case ".rtf":
                      textBox.LoadFile (n.FilePath);
                      tabControl.SelectedTab = textTab;
                      break;

                  case ".bmp":
                  case ".jpg":
                  case ".jpeg":
                  case ".gif":
                  case ".png":
                      picture = new Bitmap(n.FilePath);
                      pictureBox_SizeChanged(null, null);
                      tabControl.SelectedTab = pictureTab;
                      break;

                  case ".htm":
                  case ".html":
                      htmlBox.Url = new Uri("file://" + n.FilePath);
                      tabControl.SelectedTab = htmlTab;
                      break;

                  case ".csv":
                      readDataFile(n.FilePath);
                      tabControl.SelectedTab = dataTab;
                      break;

                  case ".xml" :
                      readXmlFile(n.FilePath);
                      tabControl.SelectedTab = xmlTab;
                      break;
              }
          }
      }

      private void readXmlFile(string filePath)
      {
          TreeNode current = null;
          // using System.Xml;
          XmlTextReader f = new XmlTextReader (filePath);
          while (f.Read())
          {
              if (f.NodeType == XmlNodeType.Element)
              {
                  TreeNode n = new TreeNode();
                  n.Text = f.Name;
                  if (current == null)
                      xmlBox.Nodes.Add(n);
                  else
                      current.Nodes.Add(n);
                  current = n;
              }
              else if (f.NodeType == XmlNodeType.EndElement)
              {
                  current = current.Parent;
              }
          }
      }

      private void readDataFile (string filePath)
      {
          dataBox.Rows.Clear();
          dataBox.ColumnCount = 0;

          StreamReader f = new StreamReader(filePath);
          string txt = f.ReadLine();

          while (txt != null)
          {
              string [] pole = txt.Split (',');
              
               if (dataBox.ColumnCount < pole.Length)
                  dataBox.ColumnCount = pole.Length;

               dataBox.Rows.Add(pole);
 
               txt = f.ReadLine();
          }

          /*
          string txt = "";
          List <string> list = new List <string> ();

          StreamReader f = new StreamReader(filePath);
          int n = f.Read();
          while (n >= 0)
          {
              char c = (char)n;
              if (c == ',')
              {
                  list.Add(txt);
                  txt = "";
              }
              else if (c >= ' ')
                  txt += c;
              else if (c == '\n')
              {
                  if (dataBox.ColumnCount < list.Count)
                      dataBox.ColumnCount = list.Count;
                  dataBox.Rows.Add (list.ToArray ());
                  list.Clear ();
                  txt = "";
              }

              n = f.Read();
          }
           */

          /*
          string txt = "";
          string[] pole = new string [] { };
          int cnt = 0;

          StreamReader f = new StreamReader(filePath);
          int n = f.Read ();
          while (n >= 0)
          {
              char c = (char) n;
              if (c == ',')
              {
                  string[] nove = new string[cnt + 1];
                  for (int i = 0; i < cnt; i++)
                      nove[i] = pole[i];
                  nove[cnt] = txt;
                  pole = nove;
                  cnt ++;
                  txt = "";
              }
              else if (c >= ' ')
                  txt += c;
              else if (c == '\n')
              {
                  if (dataBox.ColumnCount < cnt)
                      dataBox.ColumnCount = cnt;
                  dataBox.Rows.Add (pole);
                  txt = "";
                  cnt = 0;
                  pole = new string [] { };
              }

              n = f.Read();
          }
          */
      }

      private void showStatus(string s)
      {
         statusField.Text = s;
      }

      private void treeView_BeforeSelect (object sender, TreeViewCancelEventArgs e)
      {
         MyTreeNode n = e.Node as MyTreeNode;
         addSubdirs (n, 2);
         // showStatus ("Before select " + n.Path);
      }

      private void treeView_BeforeExpand (object sender, TreeViewCancelEventArgs e)
      {
         MyTreeNode n = e.Node as MyTreeNode;
         addSubdirs (n, 2);
         // showStatus ("Before expand " + n.Path);
      }

      private void treeView_AfterSelect (object sender, TreeViewEventArgs e)
      {
         MyTreeNode n = e.Node as MyTreeNode;
         showFiles (n);
         // showStatus ("Selected " + n.Path);
      }

      private void listView_Click(object sender, EventArgs e)
      {
          if (listView.SelectedItems.Count != 0)
          {
              MyListNode n = listView.SelectedItems[0] as MyListNode;
              showFile (n);
          }
      }

      private void pictureBox_SizeChanged (object sender, EventArgs e)
      {
          if (picture != null)
          {
              int w0 = picture.Width;
              int h0 = picture.Height;
              int w = pictureBox.Width;
              int h = pictureBox.Height;
              int h1 = (int) ( (float)(h0) * w / w0 );
              if (h > h1)
                  h = h1;
              else
                  w = (int) ( (float)(w0) * h / h0 );

              pictureBox.Image = new Bitmap (picture, w, h);
          }
      }


   }
}
